#
# Copyright 2014 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
#
# Refer to the README and COPYING files for full details of the license
#

from testrunner import VdsmTestCase as TestCaseBase
from vdsm import qemuImg
from vdsm import utils


class FakeExecCmd(object):

    def __init__(self, *calls):
        self.calls = list(calls)
        self.saved = None

    def __call__(self, cmd, **kw):
        call = self.calls.pop(0)
        return call(cmd, **kw)

    def __enter__(self):
        self.saved = utils.execCmd
        utils.execCmd = self

    def __exit__(self, t=None, v=None, tb=None):
        utils.execCmd = self.saved


class QemuimgCreateTests(TestCaseBase):

    def test_no_format(self):

        def create_no_format(cmd, **kw):
            assert cmd == [qemuImg._qemuimg.cmd, 'create', 'image']
            return 0, '', ''

        with FakeExecCmd(create_no_format):
            qemuImg.create('image')

    def test_qcow2_compat_not_supported(self):

        def qcow2_compat_not_supported(cmd, **kw):
            assert cmd == [qemuImg._qemuimg.cmd, 'create', '-f', 'qcow2', '-o',
                           '?', '/dev/null']
            return 0, 'Supported options:\nsize ...\n', ''

        def create_qcow2_no_compat(cmd, **kw):
            assert cmd == [qemuImg._qemuimg.cmd, 'create', '-f', 'qcow2',
                           'image']
            return 0, '', ''

        with FakeExecCmd(qcow2_compat_not_supported, create_qcow2_no_compat):
            qemuImg.create('image', format='qcow2')

    def test_qcow2_compat_supported(self):

        def qcow2_compat_supported(cmd, **kw):
            assert cmd == [qemuImg._qemuimg.cmd, 'create', '-f', 'qcow2', '-o',
                           '?', '/dev/null']
            return 0, 'Supported options:\ncompat ...\n', ''

        def create_qcow2_compat(cmd, **kw):
            assert cmd == [qemuImg._qemuimg.cmd, 'create', '-f', 'qcow2', '-o',
                           'compat=0.10', 'image']
            return 0, '', ''

        with FakeExecCmd(qcow2_compat_supported, create_qcow2_compat):
            qemuImg.create('image', format='qcow2')
